/* ctcd.c: menu for c't Linux based AV CD-ROM Knoppicillin 4 */

/* (C) 2002, 2003, 2004, 2005 Oliver Diedrich odi@ct.heise.de */
/* c't Magazin fuer Computertechnik / c't magazine for computer technology */
/* This file may be redistributed under the terms of the */
/* GNU Public License (GPL), see www.gnu.org */


/*
what we need:
-------------
* linux must provide writeable directories /mnt, /tmp, /ramdisk
* linux must start this program from init
* some binaries: sh, cp, mv, ls, less, gunzip, losetup, mount, lspci, dmesg,
  /sbin/reboot, /usr/bin/network-setup
* maybe some other details ...
*/




#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <termios.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <ctype.h>
#include <errno.h>
#include <sys/mount.h>
#include <limits.h>

#include "ctcd.h"

int check[N_CHECK] = { 0x01, 0x04, 0x06, 0x07, 0x0b, 0x0c, 0x0e,
                       0x11, 0x14, 0x16, 0x17, 0x1b, 0x1c, 0x1e,
		       0x83 };


/* global flags */
int term_status = 0;          /* 1: terminal in non echo non canonical mode */

/* global AV program related information */
avprog_t *avprog[MAX_AVPROG]; /* AV programs read from AVPROG_CONFIG */
int n_avprog;                 /* # AV progs read from AVPROG_CONFIG */
int n_av_selected;            /* # selected AV progs */
int partitions_checked = 0;   /* # partitions checked so far */



#define MAIN_OPT 5            /* main menu */
char *main_opt[] = { "Virentest ...",             /* 1 from menu() */
                     "Systeminformationen ...",   /* 2 from menu() */
                     "Einstellungen ...",         /* 3 */
                     "Mensteuerung neu starten", /* 4 from menu() */
                     "Rechnerneustart" };         /* 0 from menu() */

#define SYS_OPT 6             /* system information menu */
char *sys_opt[] = { "System",
                    "Massenspeicher",
                    "Kernel-Bootmeldungen (Q bringt Sie ins Men zurck)",
                    "Dateien verwalten (F10 bringt Sie ins Men zurck)",
                    "Shell (Strg-D bringt Sie ins Men zurck)",
                    "zurck" };

#define AV_OPT 6              /* virus scan menu */
char *av_opt[] = { "Virensignaturen aus dem Internet aktualisieren",
/* not implemented
                   "Virensignaturen von Windows- oder DOS-Laufwerk aktualisieren",
*/
                   "Computer auf Viren testen ...",
                   "Viren entfernen ...",
                   "Reports ansehen",
                   "Reports auf Diskette kopieren",
                   "zurck" };

#define TEST_OPT 4            /* test for viruses menu */
char *test_opt[] = { "Virentest ber alle Laufwerke starten",
                     "einzelne Laufwerke auf Viren testen ...",
                     "Verzeichnis /mnt auf Viren testen",
                     "zurck" };

#define DISINF_OPT 4          /* disinfect menu */
char *disinf_opt[] = { "Viren von allen Laufwerken entfernen",
                       "einzelne Laufwerke subern ...",
                       "Verzeichnis /mnt subern",
                       "zurck" };

#define SET_OPT 4
char *set_opt[] = { "Netzwerk konfigurieren",
                    "Proxy konfigurieren",
                    "Antivirenprogramme auswhlen ...",
                    "zurck" };



partition_t *partition_list[MAX_PARTITIONS]; /* readable partitions */
char *partition_name[MAX_PARTITIONS];        /* selection list with the names */
                                             /* of the readable partitions */
                                             /* for menu() */
int n_partition_list;                        /* # readable partitions */




/********************/
/* helper functions */
/********************/


void clear()
/* clears the screen */
{
  write(STDOUT_FILENO, CLEAR, strlen(CLEAR));
}



void wait_for_key()
/* asks the user to press a key */
{
  if (term_status) printf("\nBitte drcken Sie eine Taste: ");
  else printf("Bitte drcken Sie ENTER: ");
  fflush(stdout);
  (void) getchar();
  if (term_status) printf("\r                                \r");
}



char *mkstr(char str[])
/* allocates memory for string str, copies str to it, and removes '\n' at end */
/* NULL: error, else: copied string */
{
  char *c;
  int l = strlen(str);
  c = malloc(l+1);
  if (!c) return NULL;
  memset(c, 0, l+1);
  strncpy(c, str, l);
  if ('\n' == c[l-1]) c[l-1] = '\0';
  return c;
} /* mkstr() */



int menu(char *title, char *options[], int n_options)
/* displays a menu with the entries in options according to term_status */
/* last entry in options is interpreted as "quit menu" */
/* 0: quit, else: action to perform */
/* user selects options[0] -> 1 */
/* user selects options[1] -> 2 */
/* user selects last option (options[n_options-1]) -> 0 */
{
  int i;
  int in = 0;
  int t = 0;
  char str[80];

  if (term_status)
  {
    while (in != '\n')
    {
      clear();
#ifdef TEST
  printf("TEST");
#endif
      printf(" %s%s%s\n\n", BOLD, title, NORMAL);

      for (i=0; i<n_options; i++)
      {
        printf(" %s %s %s\n", (t==i) ? REVERSE : "", options[i], 
                           (t==i) ? NORMAL : "");
      }
      printf("\n");
      in = getchar();
      if (ESC_KEY == in) 
      {
        in = getchar();
        if (CTRL_KEY == in)
        {
          in = getchar();
          if (CURS_DN == in)
          {
            if (t<(n_options-1)) t++;
            else t = 0;
          }
          else if (CURS_UP == in)
          {
            if (t>0) t--;
            else t = n_options-1;
          }
        }
      } /* if ESC_KEY */

#ifdef TEST
  if ('q' == in) return 0;    /* quit menu with 'q' */
#endif

    } /* while */
    if ((n_options-1) == t) return 0;   /* quit */
    else return t+1;
  } /* if (term_status) */

  else
  {
    clear();
    printf("%s\n\n", title);
    for (i=0; i<n_options; i++) printf("%d  %s\n", i+1, options[i]);
    while ( (in<1) || (in>n_options) )
    {
      printf("\nBitte whlen Sie (1-%d): ", n_options);
      fflush(stdout);
      fgets(str, 80, stdin);
      in = atoi(str);
    }
    if (n_options == in) return 0;
    else return in;
  } /* else */
} /* menu() */




/*********************/
/* termios functions */
/*********************/

int init_terminal()
/* sets terminal to non echo and non canonical mode */
/* 0: error, 1: ok */
{
  struct termios tios;

  if (tcgetattr(STDIN_FILENO, &tios)) return 0;
  tios.c_lflag &= ~ECHO;
  tios.c_lflag &= ~ICANON;
#ifndef TEST
  tios.c_lflag &= ~ISIG;      /* disables Ctrl-C */
#endif
  if (tcsetattr(STDIN_FILENO, TCSAFLUSH, &tios)) return 0;
  return 1;
} /* init_terminal() */



int restore_terminal()
/* sets terminal to echo and canonical mode */
/* 0: error, 1: ok */
{
  struct termios tios;

  clear();  
  if (tcgetattr(STDIN_FILENO, &tios)) return 0;
  tios.c_lflag |= ECHO;
  tios.c_lflag |= ICANON;
#ifndef TEST
  tios.c_lflag |= ISIG;
#endif
  if (tcsetattr(STDIN_FILENO, TCSANOW, &tios)) return 0;
  return 1;
} /* restore_terminal */




/******************/
/* init functions */
/******************/


void update(char *proxy)
/* checks ftp.heise.de for updates */
{
  char str[256];

  if (proxy)
  {
    sprintf(str, "export http_proxy=%s ; %s", proxy, UPDATE);
    system(str);
  }
  else system(UPDATE);
} /* update() */




/****************************/
/* system related functions */
/****************************/


int open_shell()
/* forks, opens a shell in child, waits for end of child */
/* 0: error, 1: ok */
{
  pid_t pid;

  pid = fork();
  if (-1 == pid) return 0;
  if (pid)
  {
    if (-1 == wait(NULL))
    {
      perror("wait()");
      return 0;
    }
  }
  else          /* child process */
  {
    if (term_status) restore_terminal();
    if (-1 == execl("/bin/bash", "/bin/bash", NULL))
    {
      if (term_status) init_terminal();
      return 0;
    }
  }
  if (term_status) init_terminal();
  return 1;
} /* open_shell() */




int start_mc()
/* forks, starts mc in child, waits for end of child */
/* 0: error, 1: ok */
{
  pid_t pid;

  pid = fork();
  if (-1 == pid) return 0;
  if (pid)
  {
    if (-1 == wait(NULL))
    {
      perror("wait()");
      return 0;
    }
  }
  else          /* child process */
  {
    if (term_status) restore_terminal();
    if (-1 == execl("/usr/bin/mc", "/usr/bin/mc", "/", NULL))
    {
      if (term_status) init_terminal();
      return 0;
    }
  }
  if (term_status) init_terminal();
  return 1;
} /* start_mc() */




/*********************************/
/* av software related functions */
/*********************************/


int select_avprog(avprog_t *avprog[], int n_avprog, int first)
/* displays a menu with the entries in avprog with the option to select or */
/* deselect entries via blank key. avprog[]->sel is set accordingly */
/* returns # of selected entries */
{
  int i;
  int in = 0;
  int t = n_avprog;

  while (in != '\n')
  {
      clear();
#ifdef TEST
  printf("TEST");
#endif
    printf(" %s%s%s\n\n", BOLD, VERSION, NORMAL);

    printf(" Welche Antiviren-Programme sollen verwendet werden?\n");
    printf(" Als Voreinstellung sind alle Programme ausgewhlt.\n\n");
    printf(" Mit den Pfeiltasten bewegen Sie sich im Men, mit der Leertaste\n");
    printf(" oder der Return-Taste knnen Sie ein Programm an- oder abwhlen.\n\n");

    for (i=0; i<n_avprog; i++)
    {
      printf(" %s%s%s%s %s %s %s\n", BOLD, UNDERLINE,
             avprog[i]->selected ? "X" : " ", NORMAL,
             (t==i) ? REVERSE : "",
             avprog[i]->name, (t==i) ? NORMAL : "");
    }
    printf("   %s weiter %s\n\n\n", (t==n_avprog) ? REVERSE  : "", 
           (t==n_avprog) ? NORMAL : "");

    if (first)
    {
      printf("%s Der Einsatz der enthaltenen Antiviren-Software\n", BOLD);
      printf(" ist nur im Rahmen der Knoppicillin-CD gestattet.\n\n%s",
             NORMAL);
    }

    in = getchar();
    if ('\n' == in)
    {
      if ((n_avprog) == t) 
      {
        t = 0;
        for (i=0; i<n_avprog; i++)
        {
          if (avprog[i]->selected) t++;
        }
        return t;
      }
      else in = ' ';
    }
      
    if (' ' == in)
    {
      if (t<n_avprog)
      {
        if (avprog[t]->selected) avprog[t]->selected = 0;
        else avprog[t]->selected = 1;
      }
    }
    if (ESC_KEY == in) 
    {
      in = getchar();
      if (CTRL_KEY == in)
      {
        in = getchar();
        if (CURS_DN == in)
        {
          if (t<(n_avprog)) t++;
          else t = 0;
        }
        else if (CURS_UP == in)
        {
          if (t>0) t--;
          else t = n_avprog;
        }
      }
    } /* if ESC_KEY */
  } /* while */
  return 0;         /* should never be reached */
} /* select_avprog() */



int update_av_drive(avprog_t *avprog[])
/* copies and unpacks virus signatures from a partition */
/* 0: no files copied; else: number of updated AV signatures */
{
  int action = 0;
  int mounted = 0;
  int count = 0;
  int av;
  char part_name[20];

  memset(part_name, 0, 20);
  action = menu("Whlen Sie das Laufwerk mit den Virensignaturen:",
           partition_name, n_partition_list);
  if (action)       /* 1 if partition_list[0] selected, */
  {                 /* 2 for partition_list[1], and so on */
    clear();
    sprintf(part_name, "/dev/%s", partition_list[action-1]->name);
    if (-1 != mount(part_name, MNT, "vfat", MS_RDONLY, NULL)) mounted = 1;
    else if (-1 != mount(part_name, MNT, "ntfs", MS_RDONLY, NULL)) mounted = 1;
    else if (-1 != mount(part_name, MNT, "msdos", MS_RDONLY, NULL)) mounted = 1;
    else if (-1 != mount(part_name, MNT, "iso9660", MS_RDONLY, NULL))
     mounted = 1;
    else if (-1 != mount(part_name, MNT, "ext3", MS_RDONLY, NULL)) mounted = 1;
    else if (-1 != mount(part_name, MNT, "ext2", MS_RDONLY, NULL)) mounted = 1;
    else if (-1 != mount(part_name, MNT, "reiserfs", MS_RDONLY, NULL))
     mounted = 1;
    else if (-1 != mount(part_name, MNT, "xfs", MS_RDONLY, NULL)) mounted = 1;

    if (mounted)              /* drive mounted */
    {
      for (av=0; av<n_avprog; av++)
      {
        if (avprog[av]->selected)
        {
          if (!av_update_disk(avprog[av], MNT))
          {
            printf("\nDie Signaturdateien fr Antivirenprogramm %s\n",
                   avprog[av]->name);
            printf("konnten nicht von %s aktualisiert werden.\n\n", 
                   partition_name[action-1]);
	    wait_for_key();
          }
          else count++;
        }
      } /* for av */
      umount(MNT);
    }
    else
    {
      printf("\nDas Laufwerk lsst sich nicht mounten. Sorry.\n");
      wait_for_key();
      return 0;
    }
  }
  return(count);
} /* update_av_drive() */



int paint_symbol(int i)
/* paints a growing dot */
/* returns # of current symbol which should be given as argument i on */
/* next call */
{
  switch (i)
  {
    case 0:
      fputs("\r.\r", stdout);
      fflush(stdout);
      i++;
    break;
    case 1:
      fputs("\ro\r", stdout);
      fflush(stdout);
      i++;
    break;
    case 2:
      fputs("\rO\r", stdout);
      fflush(stdout);
      i++;
    break;
    case 3:
      fputs("\ro\r", stdout);
      fflush(stdout);
      i++;
    break;
    case 4:
      fputs("\r.\r", stdout);
      fflush(stdout);
      i++;
    break;
    case 5:
      fputs("\r \r", stdout);
      fflush(stdout);
      i=0;
    break;
  }
  return i;
} /* paint_symbol() */




int check_path(avprog_t *avprog, char *path, char *part_name, int disinfect)
/* runs av on directory path; disinfect ? desinfizieren : nur checken */
/* if (part_name != NULL), part_name is used as name for report file (called */
/* by check_partition() when single drives or all drives are tested); */
/* else global variable partition_checked is used as name for report (called */
/* directly from antivir() via "check /mnt" function */
/* 0: path was not checked, 1: path was checked */
{
  char str[4096];
  int status;

  memset(str, 0, 4096);
  if (disinfect) status = av_disinfect(avprog, path);
  else  status = av_check(avprog, path);
  if (!status)
  {
    printf("\nDer %s-Lauf ist mit einem Fehler abgebrochen.\n", avprog->name);
    wait_for_key();
    return 0;
  }
  else
  {
    if (part_name)
     sprintf(str, "mv -f %s/%s %s/%s.report.%s.%s", avprog->wd, avprog->log, 
                   AV_PATH, avprog->name, disinfect ? "disinfect" : "scan",
                   part_name);
    else 
     sprintf(str, "mv -f %s/%s %s/%s.report.%s.%03d", avprog->wd, avprog->log, 
                   AV_PATH, avprog->name, disinfect ? "disinfect" : "scan",
                   partitions_checked+1);
    status = system(str);
    if WEXITSTATUS(status)
    {
      printf("\nDie Ergebnisse von %s lieen sich nicht sichern.\n", 
             avprog->name);
      wait_for_key();
      return 0;
    }
    else return 1;
  }
} /* check_path() */




int check_partition(partition_t *part, int disinfect)
/* mounts partition part and starts check_path() on it */
/* 0: partition was not checked, 1: partition was checked */
{
  int i, av, checked;
  int mounted=0;
  char part_name[20];
  char str[256];
  unsigned long mountflags;
  int status;

  memset(part_name, 0, 20);
  sprintf(part_name, "/dev/%s", part->name);

/*
  if (disinfect)
  {
    sprintf(str, "insmod %s", UFSD);
    system(str);
  }
*/
  if (disinfect) mountflags = 0; else mountflags = MS_RDONLY;
  checked = 0;
  clear();
  for (i=0; i<N_CHECK; i++)
  {
    if (check[i] == part->id)       /* Windows/DOS partition */
    {
      if (-1 != mount(part_name, MNT, "vfat", mountflags, NULL))
       mounted = 1;
      else if (-1 != mount(part_name, MNT, "msdos", mountflags, NULL))
       mounted = 1;
      else if (-1 != mount(part_name, MNT, "iso9660", mountflags, NULL))
       mounted = 1;
      else if (-1 != mount(part_name, MNT, "ext3", mountflags, NULL))
       mounted = 1;
      else if (-1 != mount(part_name, MNT, "ext2", mountflags, NULL))
       mounted = 1;
      else if (-1 != mount(part_name, MNT, "reiserfs", mountflags, NULL))
       mounted = 1;
      else if (-1 != mount(part_name, MNT, "xfs", mountflags, NULL))
       mounted = 1;
      else
      {
        if (disinfect)
        { 
/*
          if (-1 != mount(part_name, MNT, "ufsd", mountflags, NULL))
           mounted = 1;
*/
          sprintf(str, "mount -rw -t ntfs %s %s", part_name, MNT);
          status = system(str);
          if (! WEXITSTATUS(status)) mounted = 2;
        }
        else
        { 
          if (-1 != mount(part_name, MNT, "ntfs", mountflags, NULL))
           mounted = 1;
        }
      }

      if (mounted)
      {
        printf("\n%sPartition %s wird %s.%s\n\n", BOLD, part->name, 
               disinfect ? "von Viren gereinigt" : "auf Viren berprft",
               NORMAL);
        printf("Dieser Vorgang kann einige Zeit in Anspruch nehmen.\n");
        printf("Bitte haben Sie Geduld, bis Knoppicillin meldet, dass der Scan beendet ist.\n");
        for (av=0; av<n_avprog; av++)
        {
          if (avprog[av]->selected)
          {
            printf("\n");
            if (!check_path(avprog[av], MNT, part->name, disinfect))
             printf("\nAufruf von %s fehlgeschlagen.\n", avprog[av]->name);
            else checked = 1;
          }
        } /* for av */
        if (checked) printf("\n\nPartition %s %s.\n", part->name,
                            disinfect ? "desinfiziert" : "berprft");
        if (1 == mounted) umount(MNT);
        else
        {
          sprintf(str, "umount %s", MNT);
          system(str);
        }
        wait_for_key();
        return checked;
      }  /* if (mounted) */
      else
      {
        printf("\nDie Partition %s lie sich nicht mounten.\n", part->name);
        wait_for_key();
        return 0;
      }
    } /* if (check[i]) */
  } /* for i */
  return 0;
} /*  check_partition() */



int run_av(disk_t *disk[], int n_disk, int disinfect)
{
/* check (disinfect=0) or check (disinfect=1) all partitions */
/* 0: no partitions checked, else: # partitions checked */

  char str[256];
  int i;
  partition_t *part;
  int n = 0;

  clear();
  memset(str, 0, 255);

  for (i=0; i<n_disk; i++)              /* check partitions of all disks */
  {
    part = disk[i]->first;
    while (part)                        /* device (still) has partitions */
    {
      n += check_partition(part, disinfect);
      part = part->next;
    }
  }

  if (n)
  {
    printf("\n\n%d %s %s.\n", n, 1==n ? "Partition" : "Partitionen",
           disinfect ? "von Viren gereinigt" : "auf Viren berprft");
    printf("Drcken Sie die Taste 'q', um die Anzeige der Ergebnisse abzubrechen.\n");
    printf("Mit ':n' schalten Sie zur nchsten Protokolldatei weiter.\n");
  }
  else printf("Keine lesbaren Partitionen gefunden.\n");
  wait_for_key();
  printf("\n");
  if (n)
  {
    sprintf(str, "less %s/*.report.*", AV_PATH);
    system(str);
  }
  return n;
} /* run_av() */




int build_partition_list(disk_t *disk[], int n_disk)
/* builds a list of readable partitions in global variable partition_list; */
/* partition_list[0] contains /dev/fd0 */
/* names of readable partitions for menu() are stored in partition_name */
/* 0: error/no partitions available, else: # partitions in part_check[] */
{
  int i, j;
  int n;
  partition_t *part;

  partition_name[0] = mkstr("Diskette (fd0)");
  if (partition_list[0]) free(partition_list[0]);
  partition_list[0] = malloc(sizeof(partition_t));
  partition_list[0]->name = mkstr("fd0");
  n = 1;

  for (i=0; i<n_disk; i++)              /* check partitions on all disks */
  {
    part = disk[i]->first;
    while (part)                        /* device (still) has partitions */
    {
      for (j=0; j<N_CHECK; j++)
      {
        if (check[j] == part->id)       /* Windows/DOS partition */
        {
          partition_list[n] = part;
          partition_name[n] = mkstr(part->name);
          n++;
          if (MAX_PARTITIONS-1 == n) 
          {
            partition_name[n] = "zurck";
            n++;
            return n;
          }
        }
      }
      part = part->next;
    }
  }
  partition_name[n] = "zurck";
  n++;
  return n;
} /* build_partition_list() */




void select_partition(int disinfect)
/* displays a selection list with partitions that can be checked for viruses */
/* and checks/disinfects the selected partition */
/* build_partition_list() must have been called before, else global variables */
/* partition_list[] and partition_name[] are not defined */
{
  int action = 0;

  while (1)
  {
    action = menu("Whlen Sie ein Laufwerk:", partition_name, n_partition_list);
    if (action)     /* 1 for partition_list[0], 2 for partition_list[1], etc. */
    {
      clear();
      if (check_partition(partition_list[action-1], disinfect)) 
       partitions_checked++;
      action = 0;
    }
    else return;              /* quit selected */
  } /* while (1) */
} /* select_partition() */




void copy_report()
/* copies *.report.* files in AV_PATH to floppy */
{
  char str[80];
  FILE *p;
  char *c;
  int found = 0;

  memset(str, 0, 80);
  clear();

  sprintf(str, "ls %s/*.report.*", AV_PATH);
  p = popen(str, "r");
  if (p)
  {
    while (fgets(str, 80, p))
    {
      c = str;
      while (strlen(c) > 6)
      {
        if (0 == strncmp(c, "report", 6)) found = 1;
        c++;
      }
    }
  }
  pclose(p);
  if (!found)
  {
    printf("\nKeine Report-Dateien gefunden.\n");
    printf("Ist schon ein Virencheck durchgelaufen?\n");
    wait_for_key();
    return;
  } /* if (p) */
          
  printf("Bitte legen Sie eine leere, formatierte Diskette ein.\n\n");
  printf("Falls Sie die Diskette noch formatieren mssen:\n");
  printf("Legen Sie eine leere Diskette ein, wechseln Sie mit Alt-F2 auf die Shell,\n");
  printf("und geben Sie dort ein:\n\n");
  printf("mkdosfs /dev/fd0\n\n");
  printf("Anschlieend kommen Sie mit Alt-F1 wieder hierhin zurck.\n");
  wait_for_key();

  if (-1 == mount("/dev/fd0", MNT, "vfat", 0, NULL))
  {
    perror("mount /dev/fd0");
    printf("\nDie Diskette lsst sich nicht mounten. Sorry.\n");
    wait_for_key();
  }
  else
  {
    printf("\nDie folgenden Dateien werden auf Diskette kopiert:\n");
    sprintf(str, "ls %s/*.report.*", AV_PATH);
    system(str);
    sprintf(str, "cp %s/*.report.* %s", AV_PATH, MNT);
    system(str);
  }
  umount(MNT);
} /* copy_report() */



int check_viruses(disk_t *disk[], int n_disk)
/* test all drives, one particular drive, or /mnt for viruses */
{
  int action = 0;
  int av;

  while (1)
  {
    action = menu("Computer auf Viren testen", test_opt, TEST_OPT);
    switch (action)
    {
      case 0:                 /* go back to main menu */
              return 1;

      case 1:                 /* start all AV progs across all partitions */
              partitions_checked = run_av(disk, n_disk, 0);
              action = 0;
              break;

      case 2:                 /* check a single partition */
              n_partition_list = build_partition_list(disk, n_disk);
              if (n_partition_list) select_partition(0);
              else
              {
                printf("\nKeine Partitionen gefunden, die sich auf Viren testen lassen. Sorry.\n");
                wait_for_key();
              }
              action = 0;
              break;

      case 3:                 /* check MNT directory */
              clear();
              printf("\n%sDas Verzeichnis %s wird auf Viren berprft.%s\n\n",
                     BOLD, MNT, NORMAL);
              printf("Dieser Vorgang kann einige Zeit in Anspruch nehmen.\n");
              printf("Bitte haben Sie Geduld, bis Knoppicillin meldet, dass der Scan beendet ist.\n\n");
              for (av=0; av<n_avprog; av++)
              {
                if (avprog[av]->selected)
                {
                  if (check_path(avprog[av], MNT, NULL, 0)) 
                   partitions_checked++;
                  else
                  {
                    printf("\nAufruf von %s fehlgeschlagen.\n", avprog[av]->name);
                    printf("Das Verzeichnis %s wurde nicht berprft.\n", MNT);
                    wait_for_key();
                  }
                }
              }
              action = 0;
              break;

      default: printf("\nFehler in der internen Logik, Programmende\n");
              return 0;
    } /* switch */
  } /* while (1) */
} /* check_viruses() */



int disinfect(disk_t *disk[], int n_disk)
/* disinfect all drives, one particular drive, or /mnt */
{
  int action = 0;
  int av;

  clear();
  printf(" %sViren entfernen\n\n\n", BOLD);
  printf("                                 ACHTUNG!%s\n\n", NORMAL);
  printf("     Beim automatischen Desinfizieren kann es zu Datenverlusten kommen,\n");
  printf("     die ber die befallenen Dateien hinausgehen. Beispielsweise ist es \n");
  printf("     mglich, dass Mailboxen aufgrund lediglich einer infizierten Mail\n");
  printf("     komplett gelscht werden.\n\n");
  printf("     Whlen Sie im folgenden Men den Menpunkt \"zurck\" aus, wenn Sie\n");
  printf("     dieses Risiko nicht eingehen wollen.\n\n");
  wait_for_key();

  while (1)
  {
    action = menu("Viren entfernen", disinf_opt, DISINF_OPT);
    switch (action)
    {
      case 0:                 /* go back to main menu */
              return 1;

      case 1:                 /* start all AV progs across all partitions */
              partitions_checked = run_av(disk, n_disk, 1);
              action = 0;
              break;

      case 2:                 /* disinfect a single partition */
              n_partition_list = build_partition_list(disk, n_disk);
              if (n_partition_list) select_partition(1);
              else
              {
                printf("\nKeine Partitionen gefunden, die sich desinfizieren lassen. Sorry.\n");
                wait_for_key();
              }
              action = 0;
              break;

      case 3:                 /* disinfect MNT directory */
              clear();
              printf("\n%sDas Verzeichnis %s wird gereinigt.%s\n\n",
                     BOLD, MNT, NORMAL);
              printf("Dieser Vorgang kann einige Zeit in Anspruch nehmen.\n");
              printf("Bitte haben Sie Geduld, bis Knoppicillin meldet, dass der Scan beendet ist.\n\n");
              for (av=0; av<n_avprog; av++)
              {
                if (avprog[av]->selected)
                {
                  if (check_path(avprog[av], MNT, NULL, 1)) 
                   partitions_checked++;
                  else
                  {
                    printf("Aufruf von %s fehlgeschlagen.\n", avprog[av]->name);
                    printf("Das Verzeichnis %s wurde nicht berprft.\n", MNT);
                    wait_for_key();
                  }
                }
              }
              wait_for_key();
              action = 0;
              break;

      default: printf("\nFehler in der internen Logik, Programmende\n");
              return 0;
    } /* switch */
  } /* while (1) */
} /* disinfect() */



int antivir(network_t *network, disk_t *disk[], int n_disk)
/* asks for signature file, reads it (if necessary) from internet or floppy, */
/* starts AV progs. Is only called from main() if av binary is available. */
/* 0: error in internal logic, 1: ready */
{
  int action = 0;
  int av;
  char str[80];

  while (1)
  {
    action = menu("Virentest", av_opt, AV_OPT);
    switch (action)
    {
      case 0:                 /* go back to main menu */
              return 1;

      case 1:                 /* download signatures from the net */
              for (av=0; av<n_avprog; av++)
              {
                if (avprog[av]->selected)
                {
                  if (!av_update_net(avprog[av], network->proxy))
                  {
                    printf("\nDas Herunterladen der Signaturen fr %s ist fehlgeschlagen\n",
                           avprog[av]->name);
                    wait_for_key();
                  }
                }
              }
              action = 0;
              break;

/* not implemented
      case 2:                 copy signatures from Windows/DOS partition
              if ((n_partition_list = build_partition_list(disk, n_disk)))
                update_av_drive(avprog);
              else
              {
	        printf("\nKeine lesbaren Partitionen gefunden. Sorry.\n");
	        wait_for_key();
              }
              action = 0;
              break;
*/

      case 2:                 /* test for viruses */
              check_viruses(disk, n_disk);
              action = 0;
              break;

      case 3:                 /* disinfect */
              disinfect(disk, n_disk);
              break;

      case 4:                 /* view reports */
              clear();
              if (partitions_checked)
              {
                clear();
                printf("Fr jeden Virenscanner und jedes getestete oder desinfizierte Laufwerk\n");
                printf("wird eine Protokolldatei angezeigt.\n\n");
                printf("Drcken Sie die Taste 'q', um die Anzeige der Ergebnisse abzubrechen.\n");
                printf("Mit ':n' schalten Sie zur nchsten Datei weiter.\n");
                wait_for_key();
                sprintf(str, "less %s/*.report.*", AV_PATH);
                system(str);
              }
              else
              {
                printf("\nKeine Report-Dateien gefunden.\n");
                printf("Ist schon ein Virencheck durchgelaufen?\n");
                wait_for_key();
              }
              action = 0;
              break;

      case 5:                 /* copy av software report to floppy */
              copy_report();
              action = 0;
              break;

      default: printf("\nFehler in der internen Logik, Programmende\n");
              return 0;
    } /* switch */
  } /* while (1) */
} /* antivir() */




/******************/
/* menus functions */
/******************/


int systeminfo(sysinfo_t *sysinfo, network_t *network, disk_t *disk[],
               int n_disk)
/* displays system information menu */
{
  int action = 0;

  while (1)
  {
    action = menu("Systeminformationen", sys_opt, SYS_OPT);
    switch (action)
    {
      case 0:                 /* quit */
              return 1;

      case 1:                 /* basic system information */
              display_sysinfo(sysinfo, network);
              action = 0;
              break;

      case 2:                 /* disk information */
              display_disk_info(disk, n_disk);
              action = 0;
              break;

      case 3:                 /* dmesg output */
              clear();
              system("dmesg|less");
              action = 0;
              break;

      case 4:                 /* start midnight commander */
              if (!start_mc())
              {
                clear();
                printf("\nDer Midnight Commander wollte nicht starten.\nSorry.\n");
                wait_for_key();
              }
              action = 0;
              break;

      case 5:                 /* shell */
              if (!open_shell())
              {
                clear();
                printf("\nDie Shell konnte nicht geffnet werden.\nSorry.\n");
                wait_for_key();
              }
              action = 0;
              break;

      default: printf("Fehler in der internen Logik, Programmende\n");
               return 1;
    } /* switch */
  } /* while */
  return 1;
} /* systeminfo() */



int settings(network_t *network)
/* displays settings menu */
{
  int action = 0;

  while (1)
  {
    action = menu("Einstellungen", set_opt, SET_OPT);
    switch (action)
    {
      case 0:                 /* quit */
              return 1;

      case 1:                 /* basic system information */
              system("/usr/sbin/network-setup");
              action = 0;
              break;

      case 2:                 /* configure network */
              clear();
              restore_terminal();
              if (network->proxy) free(network->proxy);
              if (!ask_proxy(&network->proxy)) network->proxy = NULL;
              init_terminal();
              action = 0;
              break;

      case 3:                 /* select AV progs */
              if (n_avprog)
              {
                n_av_selected = select_avprog(avprog, n_avprog, 0);
                if (!n_av_selected)
                {
                  printf("Sie haben kein Antivirenprogram angewhlt.\n");
                  printf("Ein Virencheck wird nicht mglich sein.\n");
                  wait_for_key();
                }
              }
              else
              {
                clear();
                printf("\nEs wurden kein Antivirenprogramme gefunden.\n");
                printf("Eine Auswahl der Programme ist daher nicht mglich.\n");
                wait_for_key();
              }
              action = 0;
              break;

      default: printf("Fehler in der internen Logik, Programmende\n");
               return 1;
    } /* switch */
  } /* while */
  return 1;
} /* settings() */




/********/
/* main */
/********/


int main(int argc, char **argv)
{
  sysinfo_t sysinfo;          /* cpu, memory, speed and more */
  network_t network;          /* network information */
  disk_t *disk[MAX_DISK];     /* SCSI disk properties, SCSI cdroms don't show */
                              /* show up here! */
  int n_disk = 0;             /* # SCSI disks */
  int no_raid = 0;            /* 1: don't look for IDE RAID */
  char *err = NULL;
  int action = 0;

  if (argc > 1) no_raid = 1;
  clear();
  if (!ask_proxy(&network.proxy)) network.proxy = NULL;
  update(network.proxy);
  term_status = init_terminal();
  mkdir(AV_PATH, 0777);
  get_basic_sysinfo(&sysinfo);
  get_dmi_info(&sysinfo);
  n_disk = get_disk_info(disk, no_raid);
  clear();

  n_avprog = read_config(avprog, &err);
  if (n_avprog < 1)
  {
    printf("\n\nBeim Einlesen von %s ist ein Problem aufgetreten:\n",
           AVPROG_CONFIG);
    if (-1 == n_avprog) 
    {
      printf("%s\n", err);
      n_avprog = 0;
    }
    else printf("Kein lauffhiges Antivirenprogramm gefunden.\n");
    printf("Ein Virencheck ist nicht mglich.\n");
    wait_for_key();
    if (err) free(err);
  }

  if (n_avprog)
  {
    n_av_selected = select_avprog(avprog, n_avprog, 1);
    if (!n_av_selected)
    {
      printf("Sie haben kein Antivirenprogram ausgewhlt.\n");
      printf("Ein Virencheck wird nicht mglich sein.\n");
      wait_for_key();
    }
    else
    {
      int i;
      clear();
      for (i=0; i<n_avprog; i++)
      {
        if (avprog[i]->selected) av_version(avprog[i]);
      }
      wait_for_key();
    }
  }

  while (1)
  {
    action = menu(VERSION, main_opt, MAIN_OPT);
    switch (action)
    {
      case 0:                 /* system reboot */
#ifdef TEST
              if (term_status) restore_terminal();     /* quit program */
              return 0;                                /* instead of reboot */
#else
              if (term_status) restore_terminal();
              clear();
              system("/sbin/reboot");
#endif
              break;

      case 1:                 /* av software */
              if (n_av_selected)
              {
                if (!antivir(&network, disk, n_disk)) return 1;
              }
              else
              {
                clear();
                printf("\nEs ist kein Antivirenprogramm ausgewhlt.\n");
                printf("Ein Virentest ist daher nicht mglich.\n");
                wait_for_key();
              }
              action = 0;
              break;

      case 2:                 /* system information */
              systeminfo(&sysinfo, &network, disk, n_disk);
              action = 0;
              break;

      case 3:
              settings(&network);
              action=0;
              break;

      case 4:                 /* quit */
              if (term_status) restore_terminal();
              return 0;

      default: clear();
               if (term_status) restore_terminal();
               printf("\nFehler in der internen Logik, Programmende\n");
               return 1;
    } /* switch */
  } /* while */
  return 1;         /* should never be reached */
}
