#undef TEST
#define VERSION "c't-CD 2.1 auf Knoppicillin-4"

#include <linux/limits.h>


/* screen control escape sequences */

#define REVERSE "\033[07m"
#define UNDERLINE "\033[04m"
#define BOLD "\033[01m"
#define NORMAL "\033[m"
#define CLEAR "\33[H\33[2J"


/* cursor key codes */

#define ESC_KEY 27
#define CTRL_KEY 91
#define CURS_UP 65
#define CURS_DN 66
#define CURS_LE 68
#define CURS_RI 67


/* system info */

#define MAX_DISK 30           /* sda .. sdz */
#define MAX_PCI 200           /* # PCI devices from lspci */
#define MAX_PARTITIONS 50     /* # partitions in selection list */
                              /* for partition check */
#define N_CHECK 15            /* partition IDs to check */


/* update */

#define UPDATE "/cdrom/KNOPPIX/ct/k4update"

struct si                     /* basic system information */
{
/* from /proc/cpuinfo */
  char *cpu;
  int mhz;
  unsigned int mem;
/* DMI info */
  char *sys_vendor;           /* NULL means not set */
  char *sys_product;
  char *sys_serial;
  char *bios_vendor;
  char *bios_version;
  char *bios_release;
  char *board_vendor;
  char *board_product;
  char *board_version;
  char *board_serial;
  char *chassis;
};
typedef struct si sysinfo_t;


struct net
{
  int up;                     /* network up */
  char device[8];             /* name of device default route points to */
  char ip[16];                /* IP adress */
  char *proxy;                /* proxy */
};
typedef struct net network_t;


struct partition              /* a partition */
{
  char *name;                 /* [s|h]dXNN, e.g. hpt45x_cbijgijehgN */
  int bootable;               /* flag */
  unsigned int start;         /* in 512 Byte sectors */
  unsigned int end;
  unsigned int blocks;        /* # 1k blocks */
  int id;                     /* type  ID */
  char *system;               /* type as string */
  struct partition *next;     /* next partition on this disk */
};
typedef struct partition partition_t;


struct disk                   /* a disk */
{
  char name[NAME_MAX+1];      /* hdXX, sdXX, dXXpXX, e.g. hpt45x_cbijgijehg */
  char *media;                /* disk, cdrom, IDE-RAID ... */
  char *model;                /* name of device; not set in SCSI disks */
  unsigned int capacity;      /* in kByte */
  int n_partitions;           /* # partitions */
  partition_t *first;         /* partition list; NULL if no partitions */
  int n_stripe;               /* # disks in IDE RAID */
  char *stripe[MAX_DISK];     /* name of disks in IDE RAID */
};
typedef struct disk disk_t;


/* AV software */

#define MAX_AVPROG 8                    /* as defined in AVPROG_CONFIG */
#define AVPROG_CONFIG "ctcd.conf"       /* config file */
#define AVPROG_CONFIG_LINE_LENGHT 1024
#define N_TRIAL 10                      /* # trials for avprog.update_net */
#define AV_PATH "/tmp/av"               /* AV program reports are stored here */
#define MNT "/mnt"                      /* mount directory; change AV_OPT[4] */
                                        /* in ctcd.c accordingly! */
#define UFSD "/cdrom/KNOPPIX/ct/ufsd.ko" /* NTFS RW driver */

/* Syntax of config files:

- The file consists of several sections, each one starting with a header in
  the form [header] (which is interpreted as the name of the AV prog) and
  some statement lines.

- Each statement line consists of a keyword (see avprog_t for the names),
  a ':', and a statement. The stament must be contained in one line (no '\')
  with no more than AVPROG_CONFIG_LINE_LENGHT characters.

- The statement must be of a form that can be executed by "sh -c statement".
  It can be a a simple command, a script, or a series of commands. The return
  value of the command (last command if several commands) is evaluated. You
  usually don't need to enclose the whole statement in '"'.

- Each keyword is allowed one time per section. A section does not have to
  contain all keywords.

- Comments starts with '#'. Empty Lines as well as tabs and spaces between
  keywords and statements are allowed.
*/

struct avprog                 /* all we need to know to setup, update, and */
{                             /* run an antivirus program */
  char *name;                 /* name of program */
  char *setup;                /* script that setups the av prog */
  char *wd;                   /* working dir for update_*, version, check */
  char *update_net;           /* updates av signatures from the net */
  char *update_disk;          /* updates av signatures from a disk; */
                              /* %s is replaced by dir name */
  char *version;              /* prints out version of signature files */
  char *check;                /* run av prog (check only for infected files); */
                              /* %s is replaced by dir name */
  char *disinfect;            /* run av prog (disinfect infected files); */
                              /* %s is replaced by dir name */
  char *log;                  /* log file path name relative to wd */
  int selected;               /* 1: used for AV tests, 0: deselected by user */
};
typedef struct avprog avprog_t;



/* prototypes sysinfo.c */
void get_basic_sysinfo(sysinfo_t *si);
int get_dmi_info(sysinfo_t *si);
int ask_proxy(char **proxy);
int get_disk_info(disk_t *disk[], int no_raid);
void display_disk_info(disk_t *disk[], int n_disk);
void display_sysinfo(sysinfo_t *sysinfo, network_t *network);



/* prototypes avprog.c */
int read_config(avprog_t *avprog[], char **error);
int av_setup(avprog_t *avprog);
int av_version(avprog_t *avprog);
int av_update_net(avprog_t *avprog, char *proxy);
int av_update_disk(avprog_t *avprog, char *path);
int av_check(avprog_t *avprog, char *path);
int av_disinfect(avprog_t *avprog, char *path);


/* prototypes ctcd.c */
void clear();
void wait_for_key();
char *mkstr(char str[]);
